﻿using Org.BouncyCastle.Crypto.Digests;
using Org.BouncyCastle.Crypto.Parameters;
using Org.BouncyCastle.Math;
using Org.BouncyCastle.Math.EC;

namespace YS.SM2
{
    /// <summary>
    /// 密码
    /// </summary>
    public class Cipher
    {
        private int ct = 1;
        private ECPoint p2;
        private SM3Digest sm3keybase;
        private SM3Digest sm3c3;
        private byte[] key = new byte[32];
        private byte keyOff = 0;
        /// <summary>
        /// 重置
        /// </summary>
        private void Reset()
        {
            sm3keybase = new SM3Digest();
            sm3c3 = new SM3Digest();
            var p = p2.GetEncoded();
            sm3keybase.BlockUpdate(p, 0, p.Length);
            sm3c3.BlockUpdate(p, 0, p.Length);
            p = p2.GetEncoded();
            sm3keybase.BlockUpdate(p, 0, p.Length);
            ct = 1;
            NextKey();
        }
        /// <summary>
        /// 下一步处理
        /// </summary>
        private void NextKey()
        {
            var sm3keycur = new SM3Digest(sm3keybase);
            sm3keycur.Update((byte)(ct >> 24 & 0x00ff));
            sm3keycur.Update((byte)(ct >> 16 & 0x00ff));
            sm3keycur.Update((byte)(ct >> 8 & 0x00ff));
            sm3keycur.Update((byte)(ct & 0x00ff));
            sm3keycur.DoFinal(key, 0);
            keyOff = 0;
            ct++;
        }
        /// <summary>
        /// 初始化EC
        /// </summary>
        /// <param name="sm2">SM2代理</param>
        /// <param name="userKey">用户Key</param>
        /// <returns></returns>
        public virtual ECPoint Init_enc(SM2Provider sm2, ECPoint userKey)
        {
            var key = sm2.Ecc_Key_Pair_Generator.GenerateKeyPair();
            var ecpriv = (ECPrivateKeyParameters)key.Private;
            var ecpub = (ECPublicKeyParameters)key.Public;
            var k = ecpriv.D;
            var c1 = ecpub.Q;
            p2 = userKey.Multiply(k);
            Reset();
            return c1;
        }
        /// <summary>
        /// 加密
        /// </summary>
        /// <param name="data">需要加密的数据</param>
        public virtual void Encrypt(byte[] data)
        {
            sm3c3.BlockUpdate(data, 0, data.Length);
            for (int i = 0; i < data.Length; i++)
            {
                if (keyOff == key.Length)
                    NextKey();
                data[i] ^= key[keyOff++];
            }
        }
        /// <summary>
        /// 初始化
        /// </summary>
        /// <param name="userD">userD</param>
        /// <param name="c1">c1</param>
        public virtual void Init_dec(BigInteger userD, ECPoint c1)
        {
            p2 = c1.Multiply(userD);
            Reset();
        }
        /// <summary>
        /// 解密
        /// </summary>
        /// <param name="data">需要解密的数据</param>
        public virtual void Decrypt(byte[] data)
        {
            for (int i = 0; i < data.Length; i++)
            {
                if (keyOff == key.Length)
                    NextKey();
                data[i] ^= key[keyOff++];
            }
            sm3c3.BlockUpdate(data, 0, data.Length);
        }
        /// <summary>
        /// 最后处理
        /// </summary>
        /// <param name="c3">c3</param>
        public virtual void Dofinal(byte[] c3)
        {
            var p = p2.GetEncoded();
            sm3c3.BlockUpdate(p, 0, p.Length);
            sm3c3.DoFinal(c3, 0);
            Reset();
        }
    }
}
