﻿using Org.BouncyCastle.Crypto.Macs;
using Org.BouncyCastle.Crypto.Parameters;
using Org.BouncyCastle.Utilities.Encoders;
using System.Text;

namespace YS.SM3
{
    /// <summary>
    /// SM3加密
    /// </summary>
    public static class SM3Utils
    {
        /// <summary>
        /// SM3加密
        /// </summary>
        /// <param name="data">待加密字符串</param>
        /// <param name="key">Key</param>
        /// <returns></returns>
        public static string EncryptSM3(this string data, string key)
        {
            var msg1 = Encoding.Default.GetBytes(data);
            var key1 = Encoding.Default.GetBytes(key);
            var keyParameter = new KeyParameter(key1);
            var sm3 = new SM3Provider();
            var mac = new HMac(sm3);//带密钥的杂凑算法
            mac.Init(keyParameter);
            mac.BlockUpdate(msg1, 0, msg1.Length);
            var result = new byte[mac.GetMacSize()];
            mac.DoFinal(result, 0);
            var hexResult = Hex.Encode(result);
            return Convert.ToBase64String(hexResult);
        }
        /// <summary>
        /// SM3加密
        /// </summary>
        /// <param name="data">待加密字符串</param>
        /// <returns></returns>
        public static string EncryptSM3(this string data)
        {
            var msg = data.ToHexByte();//把字符串转成16进制的ASCII码 
            var sm3 = new SM3Provider();
            sm3.BlockUpdate(msg, 0, msg.Length);
            var md = new byte[sm3.GetDigestSize()];//SM3算法产生的哈希值大小
            sm3.DoFinal(md, 0);
            var hexResult = Hex.Encode(md);
            return Convert.ToBase64String(hexResult);
        }
        /// <summary>
        /// 字符串转16进制字节数组
        /// </summary>
        /// <param name="data"></param>
        /// <returns></returns>
        private static byte[] ToHexByte(this string data)
        {
            var msg1 = Encoding.Default.GetBytes(data);
            var hexString = BytesToHexString(msg1);
            var returnBytes = new byte[hexString.Length / 2];
            for (int i = 0; i < returnBytes.Length; i++)
                returnBytes[i] = Convert.ToByte(hexString.Substring(i * 2, 2), 10);
            return returnBytes;
        }
        /// <summary>
        /// byte[]数组转16进制字符串
        /// </summary>
        /// <param name="input">byte[]数组</param>
        /// <returns>16进制字符串</returns>
        private static string BytesToHexString(byte[] input)
        {
            var hexString = new StringBuilder(64);
            for (int i = 0; i < input.Length; i++)
            {
                hexString.Append(String.Format("{0:X2}", input[i]));
            }
            return hexString.ToString();
        }
    }
}
