﻿using Org.BouncyCastle.Utilities.Encoders;
using System.Text;

namespace YS.SM4
{
    public static class SM4Utils
    {
        public static bool hexString = false;
        /// <summary>
        /// 加密 ECB模式
        /// </summary>
        /// <param name="plainText">明文</param>
        /// <param name="secretKey">密钥</param>
        /// <returns></returns>
        public static string Encrypt_ECB(this string plainText,string secretKey)
        {
            var ctx = new SM4Context();
            ctx.isPadding = true;
            ctx.mode = SM4Provider.SM4_ENCRYPT;
            byte[] keyBytes;
            if (hexString)
                keyBytes = Hex.Decode(secretKey);
            else
                keyBytes = Encoding.Default.GetBytes(secretKey);
            var sm4 = new SM4Provider();
            sm4.sm4_setkey_enc(ctx, keyBytes);
            var encrypted = sm4.sm4_crypt_ecb(ctx, Encoding.Default.GetBytes(plainText));
            var cipherText = Encoding.Default.GetString(Hex.Encode(encrypted));
            return cipherText;
        }
        /// <summary>
        /// 解密 ECB模式
        /// </summary>
        /// <param name="cipherText">密文</param>
        /// <param name="secretKey">密钥</param>
        /// <returns></returns>
        public static string Decrypt_ECB(this string cipherText,string secretKey)
        {
            var ctx = new SM4Context();
            ctx.isPadding = true;
            ctx.mode = SM4Provider.SM4_DECRYPT;
            byte[] keyBytes;
            if (hexString)
                keyBytes = Hex.Decode(secretKey);
            else
                keyBytes = Encoding.Default.GetBytes(secretKey);
            var sm4 = new SM4Provider();
            sm4.sm4_setkey_dec(ctx, keyBytes);
            var decrypted = sm4.sm4_crypt_ecb(ctx, Hex.Decode(cipherText));
            return Encoding.Default.GetString(decrypted);
        }
        /// <summary>
        /// 加密 CBC模式
        /// </summary>
        /// <param name="plainText">明文</param>
        /// <param name="secretKey">密钥</param>
        /// <param name="iv">iv</param>
        /// <returns></returns>
        public static string Encrypt_CBC(this string plainText,string secretKey,string iv)
        {
            var ctx = new SM4Context();
            ctx.isPadding = true;
            ctx.mode = SM4Provider.SM4_ENCRYPT;
            byte[] keyBytes;
            byte[] ivBytes;
            if (hexString)
            {
                keyBytes = Hex.Decode(secretKey);
                ivBytes = Hex.Decode(iv);
            }
            else
            {
                keyBytes = Encoding.Default.GetBytes(secretKey);
                ivBytes = Encoding.Default.GetBytes(iv);
            }
            var sm4 = new SM4Provider();
            sm4.sm4_setkey_enc(ctx, keyBytes);
            var encrypted = sm4.sm4_crypt_cbc(ctx, ivBytes, Encoding.Default.GetBytes(plainText));
            var cipherText = Encoding.Default.GetString(Hex.Encode(encrypted));
            return cipherText;
        }
        /// <summary>
        /// 解密 CBC模式
        /// </summary>
        /// <param name="cipherText">密文</param>
        /// <param name="secretKey">密钥</param>
        /// <param name="iv">iv</param>
        /// <returns></returns>
        public static string Decrypt_CBC(this string cipherText, string secretKey,string iv)
        {
            var ctx = new SM4Context();
            ctx.isPadding = true;
            ctx.mode = SM4Provider.SM4_DECRYPT;
            byte[] keyBytes;
            byte[] ivBytes;
            if (hexString)
            {
                keyBytes = Hex.Decode(secretKey);
                ivBytes = Hex.Decode(iv);
            }
            else
            {
                keyBytes = Encoding.Default.GetBytes(secretKey);
                ivBytes = Encoding.Default.GetBytes(iv);
            }
            var sm4 = new SM4Provider();
            sm4.sm4_setkey_dec(ctx, keyBytes);
            var decrypted = sm4.sm4_crypt_cbc(ctx, ivBytes, Hex.Decode(cipherText));
            return Encoding.Default.GetString(decrypted);
        }
    }
}
